searchData={"items":[{"type":"module","title":"telemetry_poller","doc":"A time-based poller to periodically dispatch Telemetry events.\n\nA poller is a process start in your supervision tree with a list\nof measurements to perform periodically. On start it expects the\nperiod in milliseconds and a list of measurements to perform. Initial delay\nis an optional parameter that sets time delay in milliseconds before starting\nmeasurements:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```\ntelemetry_poller:start_link([\n    {measurements, Measurements},\n    {period, Period},\n    {init_delay, InitDelay}\n])\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```\n:telemetry_poller.start_link(\n    measurements: measurements,\n    period: period,\n    init_delay: init_delay\n)\n```\n\n<!-- tabs-close -->","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Measurements - telemetry_poller","doc":"The following measurements are supported:\n\n  * `memory` (default)\n  * `total_run_queue_lengths` (default)\n  * `system_counts` (default)\n  * `persistent_term` (default)\n  * `{process_info, Proplist}`\n  * `{Module, Function, Args}`\n\nWe will discuss each measurement in detail. Also note that the\n`telemetry_poller` application ships with a built-in poller that\nmeasures `memory`, `total_run_queue_lengths`, `system_counts`, and `persistent_term`. This takes\nthe VM measurement out of the way so your application can focus\non what is specific to its behaviour.","ref":"telemetry_poller.html#module-measurements"},{"type":"module","title":"Memory - telemetry_poller","doc":"An event emitted as `[vm, memory]`. The measurement includes all\nthe key-value pairs returned by the `erlang:memory/0` function,\ne.g. `total` for total memory, `processes_used` for memory used by\nall processes, and so on.","ref":"telemetry_poller.html#module-memory"},{"type":"module","title":"Total run queue lengths - telemetry_poller","doc":"On startup, the Erlang VM starts many schedulers to do both IO and\nCPU work. If a process needs to do some work or wait on IO, it is\nallocated to the appropriate scheduler. The measurement includes the\nfollowing keys:\n\n  * `total` - all schedulers (CPU + IO)\n  * `cpu` - CPU schedulers\n  * `io` - IO schedulers","ref":"telemetry_poller.html#module-total-run-queue-lengths"},{"type":"module","title":"System counts - telemetry_poller","doc":"The measurement includes:\n\n  * `process_count` - the number of processes currently existing at the local node\n  * `atom_count` - the number of atoms currently existing at the local node\n  * `port_count` - the number of ports currently existing at the local node","ref":"telemetry_poller.html#module-system-counts"},{"type":"module","title":"Persistent term (since 1.2.0) - telemetry_poller","doc":"An event emitted as `[vm, persistent_term]`. The measurement includes information\nabout persistent terms in the system, as returned by `persistent_term:info/0`:\n\n  * `count` - The number of persistent terms\n  * `memory` - The total amount of memory (measured in bytes) used by all persistent terms","ref":"telemetry_poller.html#module-persistent-term-since-1-2-0"},{"type":"module","title":"Process info - telemetry_poller","doc":"A measurement with information about a given process. It must be specified\nalongside a proplist with the process name, the event name, and a list of\nkeys to be included:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-process-info"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\n{process_info, [\n {name, my_app_worker},\n {event, [my_app, worker]},\n {keys, [message_queue_len, memory]}\n]}\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\n{:process_info, [\n name: my_app_worker,\n event: [my_app, worker],\n keys: [message_queue_len, memory]\n]}\n```\n\n<!-- tabs-close -->","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Custom measurements - telemetry_poller","doc":"Telemetry poller also allows you to perform custom measurements by passing\na module-function-args tuple:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-custom-measurements"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\n{my_app_example, measure, []}\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\n{MyApp.Example, :measure, []}\n```\n\n<!-- tabs-close -->\n\nThe given function will be invoked periodically and they must explicitly invoke the\n`telemetry:execute/3` function. If the invocation of the MFA fails,\nthe measurement is removed from the Poller.\n\nFor all options, see `start_link/1`. The options listed there can be given\nto the default poller as well as to custom pollers.","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Default poller - telemetry_poller","doc":"A default poller is started with `telemetry_poller` responsible for emitting\nmeasurements for `memory` and `total_run_queue_lengths`. You can customize\nthe behaviour of the default poller by setting the `default` key under the\n`telemetry_poller` application environment. Setting it to `false` disables\nthe poller.","ref":"telemetry_poller.html#module-default-poller"},{"type":"module","title":"Examples - telemetry_poller","doc":"","ref":"telemetry_poller.html#module-examples"},{"type":"module","title":"Example 1: tracking number of active sessions in web application - telemetry_poller","doc":"Let's imagine that you have a web application and you would like to periodically\nmeasure number of active user sessions.\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-example-1-tracking-number-of-active-sessions-in-web-application"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\n-module(example_app).\n\nsession_count() ->\n   % logic for calculating session count.\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\ndefmodule ExampleApp do\n  def session_count do\n    # logic for calculating session count\n  end\nend\n```\n\n<!-- tabs-close -->\n\nTo achieve that, we need a measurement dispatching the value we're interested in:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\n-module(example_app_measurements).\n\ndispatch_session_count() ->\n   telemetry:execute([example_app, session_count], example_app:session_count()).\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\ndefmodule ExampleApp.Measurements do\n  def dispatch_session_count do\n    :telemetry.execute([:example_app, :session_count], ExampleApp.session_count())\n  end\nend\n```\n\n<!-- tabs-close -->\n\nand tell the Poller to invoke it periodically:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\ntelemetry_poller:start_link([{measurements, [{example_app_measurements, dispatch_session_count, []}]).\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\n:telemetry_poller.start_link(measurements: [{ExampleApp.Measurements, :dispatch_session_count, []}])\n```\n\n<!-- tabs-close -->\n\nIf you find that you need to somehow label the event values, e.g. differentiate between number of\nsessions of regular and admin users, you could use event metadata:\n\n<!-- tabs-open -->","ref":"telemetry_poller.html#module-elixir"},{"type":"module","title":"Erlang - telemetry_poller","doc":"```erlang\n-module(example_app_measurements).\n\ndispatch_session_count() ->\n   Regulars = example_app:regular_users_session_count(),\n   Admins = example_app:admin_users_session_count(),\n   telemetry:execute([example_app, session_count], #{count => Admins}, #{role => admin}),\n   telemetry:execute([example_app, session_count], #{count => Regulars}, #{role => regular}).\n```","ref":"telemetry_poller.html#module-erlang"},{"type":"module","title":"Elixir - telemetry_poller","doc":"```elixir\ndefmodule ExampleApp.Measurements do\n  def dispatch_session_count do\n    regulars = ExampleApp.regular_users_session_count()\n    admins = ExampleApp.admin_users_session_count()\n    :telemetry.execute([:example_app, :session_count], %{count: admins}, %{role: :admin})\n    :telemetry.execute([:example_app, :session_count], %{count: regulars}, %{role: :regular})\n  end\nend\n```\n\n<!-- tabs-close -->\n\n> #### Note {: .info}\n>\n> The other solution would be to dispatch two different events by hooking up\n> `example_app:regular_users_session_count/0` and `example_app:admin_users_session_count/0`\n> functions directly. However, if you add more and more user roles to your app, you'll find\n> yourself creating a new event for each one of them, which will force you to modify existing\nevent handlers. If you can break down event value by some feature, like user role in this\nexample, it's usually better to use event metadata than add new events.\n\nThis is a perfect use case for poller, because you don't need to write a dedicated process\nwhich would call these functions periodically. Additionally, if you find that you need to collect\nmore statistics like this in the future, you can easily hook them up to the same poller process\nand avoid creating lots of processes which would stay idle most of the time.","ref":"telemetry_poller.html#module-elixir"},{"type":"function","title":"telemetry_poller.child_spec/1","doc":"Returns a child spec for the poller for running under a supervisor.","ref":"telemetry_poller.html#child_spec/1"},{"type":"function","title":"telemetry_poller.list_measurements/1","doc":"Returns a list of measurements used by the poller.","ref":"telemetry_poller.html#list_measurements/1"},{"type":"function","title":"telemetry_poller.start_link/1","doc":"Starts a poller linked to the calling process.\n\nUseful for starting Pollers as a part of a supervision tree.","ref":"telemetry_poller.html#start_link/1"},{"type":"function","title":"Options - telemetry_poller.start_link/1","doc":"The default options are:\n\n  * `{name, telemetry_poller}`\n  * `{period, timer:seconds(5)}`\n  * `{init_delay, 0}`","ref":"telemetry_poller.html#start_link/1-options"},{"type":"type","title":"telemetry_poller.init_delay/0","doc":"An init delay for the poller.","ref":"telemetry_poller.html#t:init_delay/0"},{"type":"type","title":"telemetry_poller.measurement/0","doc":"A measurement for the poller.","ref":"telemetry_poller.html#t:measurement/0"},{"type":"type","title":"telemetry_poller.option/0","doc":"An option for the poller.","ref":"telemetry_poller.html#t:option/0"},{"type":"type","title":"telemetry_poller.options/0","doc":"A list of options for the poller.","ref":"telemetry_poller.html#t:options/0"},{"type":"type","title":"telemetry_poller.period/0","doc":"A period for the poller.","ref":"telemetry_poller.html#t:period/0"},{"type":"type","title":"telemetry_poller.t/0","doc":"The reference to a poller process.","ref":"telemetry_poller.html#t:t/0"},{"type":"extras","title":"telemetry_poller","doc":"# telemetry_poller\n\n[![Test](https://github.com/kianmeng/telemetry_poller/actions/workflows/test.yml/badge.svg)](https://github.com/kianmeng/telemetry_poller/actions/workflows/test.yml)\n[![Codecov](https://codecov.io/gh/beam-telemetry/telemetry_poller/branch/master/graphs/badge.svg)](https://codecov.io/gh/beam-telemetry/telemetry_poller/branch/master/graphs/badge.svg)\n\nAllows to periodically collect measurements and dispatch them as Telemetry events.\n\n`telemetry_poller` by default runs a poller to perform VM measurements:\n\n  * `[vm, memory]` - contains the total memory, process memory, and all other keys in `erlang:memory/0`\n  * `[vm, total_run_queue_lengths]` - returns the run queue lengths for CPU and IO schedulers. It contains the `total`, `cpu` and `io` measurements\n  * `[vm, system_counts]` - returns the current process, atom and port count as per `erlang:system_info/1`\n\nYou can directly consume those events after adding `telemetry_poller` as a dependency.\n\nPoller also provides a convenient API for running custom pollers.","ref":"readme.html"},{"type":"extras","title":"Defining custom measurements - telemetry_poller","doc":"Poller also includes conveniences for performing process-based measurements as well as custom ones.","ref":"readme.html#defining-custom-measurements"},{"type":"extras","title":"Erlang - telemetry_poller","doc":"First define the poller with the custom measurements. The first measurement is the built-in `process_info` measurement and the second one is given by a custom module-function-args defined  by you:\n\n```erlang\ntelemetry_poller:start_link(\n  [{measurements, [\n    {process_info, [{name, my_app_worker}, {event, [my_app, worker]}, {keys, [memory, message_queue_len]}]},\n    {example_app_measurements, dispatch_session_count, []}\n  ]},\n  {period, timer:seconds(10)}, % configure sampling period - default is timer:seconds(5)\n  {init_delay, timer:seconds(600)}, % configure sampling initial delay - default is 0\n  {name, my_app_poller}\n]).\n```\n\nNow define the custom measurement and you are good to go:\n\n```erlang\n-module(example_app_measurements).\n\ndispatch_session_count() ->\n    % emit a telemetry event when called\n    telemetry:execute([example_app, session_count], #{count => example_app:session_count()}, #{}).\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - telemetry_poller","doc":"You typically start the poller as a child in your supervision tree:\n\n```elixir\nchildren = [\n  {:telemetry_poller,\n   # include custom measurement as an MFA tuple\n   measurements: [\n     {:process_info, name: :my_app_worker, event: [:my_app, :worker], keys: [:memory, :message_queue_len]},\n     {ExampleApp.Measurements, :dispatch_session_count, []},\n   ],\n   period: :timer.seconds(10), # configure sampling period - default is :timer.seconds(5)\n   init_delay: :timer.seconds(600), # configure sampling initial delay - default is 0\n   name: :my_app_poller}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nThe poller above has two periodic measurements. The first is the built-in `process_info` measurement that will gather the memory and message queue length of a process. The second is given by a custom module-function-args defined by you, such as below:\n\n```elixir\ndefmodule ExampleApp.Measurements do\n  def dispatch_session_count() do\n    # emit a telemetry event when called\n    :telemetry.execute([:example_app, :session_count], %{count: ExampleApp.session_count()}, %{})\n  end\nend\n```","ref":"readme.html#elixir"},{"type":"extras","title":"Documentation - telemetry_poller","doc":"See [documentation](https://hexdocs.pm/telemetry_poller/) for more concrete examples and usage\ninstructions.","ref":"readme.html#documentation"},{"type":"extras","title":"VM metrics example - telemetry_poller","doc":"","ref":"readme.html#vm-metrics-example"},{"type":"extras","title":"Erlang - telemetry_poller","doc":"Find, in `examples/telemetry_poller_vm.erl`, an example on how to retrieve to VM measurements,\nmentioned above.\n\nTo see it in action, fire up `rebar3 shell`, then\n\n```erlang\n{ok, telemetry_poller_vm} = c(\"examples/telemetry_poller_vm\").\nok = file:delete(\"telemetry_poller_vm.beam\").  % Deletes generated BEAM\nok = telemetry_poller_vm:attach().\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - telemetry_poller","doc":"Find, in `examples/TelemetryPollerVM.ex`, an example on how to retrieve to VM measurements,\nmentioned above.\n\nTo see it in action, first compile the Erlang sources with `rebar3 compile`.\n\nThen fire up `iex -pa \"_build/default/lib/*/ebin\"`, then\n\n```elixir\n{:ok, _} = Application.ensure_all_started(:telemetry_poller)\n\n[TelemetryPollerVM] = c(\"examples/TelemetryPollerVM.ex\")\n:ok = TelemetryPollerVM.attach()\n```","ref":"readme.html#elixir"},{"type":"extras","title":"Copyright and License - telemetry_poller","doc":"Copyright (c) 2025 Erlang Ecosystem Foundation.\n\ntelemetry_poller source code is released under Apache License, Version 2.0.\n\nSee [LICENSE](LICENSE) and [NOTICE](NOTICE) files for more information.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.2.0](https://github.com/beam-telemetry/telemetry_poller/tree/v1.2.0)","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Support `persistent_term` measurements.\n- Require Erlang/OTP 24+.\n\n## [1.1.0](https://github.com/beam-telemetry/telemetry_poller/tree/v1.1.0)","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add the `init_delay` configuration option. (#70)\n\n## [1.0.0](https://github.com/beam-telemetry/telemetry_poller/tree/v1.0.0)\n\nThis release marks stability of the API. The library now requires Telemetry ~> 1.0.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix disabling the default poller via application environment.","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Drop support for OTP 20.\n\n## [0.5.1](https://github.com/beam-telemetry/telemetry_poller/tree/v0.5.1)","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Support `{:global, atom()}` and `{:via, module(), term()}` names for the poller process.\n\n## [0.5.0](https://github.com/beam-telemetry/telemetry_poller/tree/v0.5.0)","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `system_counts` measurement\n\n## [0.4.1](https://github.com/beam-telemetry/telemetry_poller/tree/v0.4.1)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve docs\n- No longer add a default name to telemetry processes\n\n## [0.4.0](https://github.com/beam-telemetry/telemetry_poller/tree/v0.4.0)\n\nTelemetry Poller has been rewritten in Erlang so it can be used by the overall Erlang community.\nTherefore, the `Telemetry.Poller` module must now be accessed as `telemetry_poller`.\nA new `process_info` measurement has also been added and the `vm_measurements` and `measurements`\nkeys have been merged into a single `measurements` key for simplicity.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `:process_info` measurement","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `vm_measurements` and `measurements` have been merged into `measurements`\n- `Telemetry.Poller` has been rewritten to `telemetry_poller`\n\n## [0.3.0](https://github.com/beam-telemetry/telemetry_poller/tree/v0.3.0)\n\nThis release marks the upgrade to Telemetry 0.4.0. This means that Poller measurements can emit a map\nof values now instead of a single one, making it less \"noisy\" when it comes to number of emitted events.\n\nAll specific memory measurements have been replaced with a single `:memory` measurement sending all\nthe values that were emitted by old measurements at once.\n\n`:run_queue_lengths` VM measurement has been removed for now, as we believe that as detailed data\nas it provided is not necessary to effectively debug the system. `:total_run_queue_lengths` VM\nmeasurement has been changed so that it reports a `:total` length of run queues, length of `:cpu`\nrun queues (including dirty CPU run queue), and length of (dirty) `:io` run queue.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `:memory` VM measurement reporting all the data returned by `:erlang.memory/0` call.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `:total_run_queue_lengths` VM measurement is reporting a `:total`, `:cpu` and `:io` run queue lengths\n  now. See documentation for more details.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- `:total_memory`, `:atom_memory`, `:atom_used_memory`, `:processes_memory`, `:processes_used_memory`,\n  `:binary_memory`, `:ets_memory`, `:code_memory` and `:system_memory` VM measurements have been removed.\n  Please use the `:memory` measurement now instead.\n\n## [0.2.0](https://github.com/beam-telemetry/telemetry_poller/tree/v0.2.0)","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `:total_run_queue_lengths` and `:run_queue_lengths` memory measurements;","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `:total_run_queue_lengths` is now included in the set of default VM measurements;\n- A default Poller process, with a default set of VM measurements, is started when `:telemetry_poller`\n  application starts (configurable via `:telemetry.poller, :default` application environment).\n- VM measurements are now provided to Poller's `:vm_measurements` option. Passing atom `:default`\n  to this option makes the Poller use a default set of VM measurements;\n- Telemetry has been upgraded to version 0.3, meaning that VM measurements now use this version to\n  emit the events.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- `Telemetry.Poller.vm_measurements/0` function has been removed in favor of `:vm_measurements`\n  option.","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed the type definition of `Telemetry.Poller.measurement/0` type - no Dialyzer warnings are\n  emitted when running it on the project.\n\n## [0.1.0](https://github.com/beam-telemetry/telemetry_poller/tree/v0.1.0)","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- The Poller process periodically invoking registered functions which emit Telemetry events.\n  It supports VM memory measurements and custom measurements provided as MFAs.","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"NOTICE","doc":"Copyright (c) 2018, Chris McCord and Erlang Solutions\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"notice.html"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}